import csv
from scraper.settings import constants

def progress_bar (iteration, total, prefix = 'Processing', suffix = 'Complete', decimals = 1, length = 100, fill = '█', printEnd = "\r"): # pragma: no cover
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    # Print New Line on Complete
    if iteration == total: 
        print()

def path_contains_filename(path):
    if path[-4:] in ['.csv', '.txt']:
        return True
    return False


def register_custom_csv_dialects():
    csv.register_dialect(
        constants.CSV_OUT_DIALECTS.DEFAULT,
        delimiter="|",
        quoting=csv.QUOTE_MINIMAL
    )

